const express = require('express');
const bodyParser = require('body-parser');
const dmdb = require('dmdb'); // 达梦数据库驱动

const app = express();
const port = 8829;

// 中间件
app.use(bodyParser.json());

const dmHost = '10.51.210.4';
const dmPort = 5237;
const dmDatabase = 'ZZ_ZWBH_PLANT_PROTECTION';
const dmUser = 'ZZ_ZWBH_PLANT_PROTECTION';
const dmPassword = 'Pp20210721';
const token = 'ZZ_ZWBH_PLANT_PROTECTION';

const connectString = `dm://${dmUser}:${dmPassword}@${dmHost}:${dmPort}?autoCommit=false&loginEncrypt=false`;

//获取数据库连接 
async function getConnection() {
  try {
      return await dmdb.getConnection(connectString);
  } catch (err) {
      throw new Error("getConnection error: " + err.message);
  }
}

getConnection().then(client => {
  client.execute('SELECT 1');
  console.log('数据库连接成功');
  client.release();
}).catch(err => {
  console.error('数据库连接失败:', err);
  process.exit(1);
});

// 健康检查端点
app.get('/health', (req, res) => {
  res.status(200).json({ status: 'ok' });
});

// SQL执行端点
app.post('/execute-sql', 
  (req, res, next) => {
    // Bearer Token验证
    const token = req.headers['authorization'];
    if (!token || token !== `Bearer ${token}`) {
      console.error('Token 未授权', token);
      return res.status(401).json({ error: '未授权' });
    }
    next();
  },
  async (req, res) => {
    const { sql } = req.body;

    if (!sql) {
      return res.status(400).json({ error: '缺少SQL查询语句' });
    }

    try {
      // 验证SQL是否为SELECT查询
      const normalizedSql = sql.trim().toLowerCase();
      
      if (!normalizedSql.startsWith('select ')) {
        return res.status(403).json({ 
          error: '仅允许SELECT查询', 
          message: '出于安全考虑，只允许执行SELECT语句'
        });
      }

      console.log(`[${new Date().getTime()}] 执行查询:`, sql);

      // 获取数据库连接
      const client = await getConnection();
      
      try {
        // 执行查询
        const result = await client.execute(sql);
        res.status(200).json({
          success: true,
          rows: result.rows,
          rowCount: result.rowsAffected,
          fields: result.name
        });
      } finally {
        // 释放连接回连接池
        client.release();
      }
    } catch (err) {
      console.error('查询执行失败:', err);
      res.status(500).json({
        error: '查询执行失败',
        message: err.message,
        detail: process.env.NODE_ENV === 'development' ? err.stack : undefined
      });
    }
  });

// 错误处理中间件
app.use((err, req, res, next) => {
  console.error('应用错误:', err);
  res.status(500).json({ 
    error: '服务器内部错误',
    message: err.message 
  });
});

// 启动服务器
app.listen(port, () => {
  console.log(`服务已启动，监听端口: ${port}`);
});

// 优雅关闭
process.on('SIGINT', async () => {
  console.log('正在关闭应用...');
  try {
    process.exit(0);
  } catch (err) {
    console.error('关闭出错:', err);
    process.exit(1);
  }
});

module.exports = app;